// 
// Transient Zone ADL.
// Based on an idea by kprsa at Forex Factory.
// This version copyright (C) 2015, NorthTrader.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
	/// TZADL: an adaption of Chaikin's Accumulation/Distribution Line, which is based on mid-bar transient zones rather than volume.
    /// </summary>
    [Description("TZADL: an adaption of Chaikin's Accumulation/Distribution Line, which is based on mid-bar transient zones rather than volume.")]
    public class TZADL : Indicator
    {
        #region Variables

		//Parameter defaults
		int iBarPeriod = 1;
		int iTZh = 1;
		bool bUseTZk = false;

		//Other
		double dSubTotal;
		bool bFirst, bSameAsChart;
		int iTZMid, iTZLeft, iTZRight, iTZMidPrev, iTZMidNext;

		#endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			CalculateOnBarClose = true;	 //Not updated on every tick
            Add(new Plot(Color.DarkRed, PlotStyle.Line, "TZAD"));
			if (iBarPeriod > 0) Add(PeriodType.Minute, iBarPeriod);  //Add sub data series if necessary
			bFirst = true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick) if CalculateOnBarClose = false. Otherwise called on bar close.
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (bFirst) SetVars();

			if (BarsInProgress == 0)
			{
				//Main data series (bars shown on chart)
				if (bSameAsChart)
				{
					//Sub and Main are the same, so only process Main
					if (CurrentBar < iTZLeft)
					{
						//CurrentBar=0 for first bar of chart. Need TZ-width number of bars to begin calculating TZADL.
						TZAD.Set(0);
						return;
					}
					else TZAD.Set(iTZh, TZAD[iTZh + 1] + iBarValue());  //Update TZADL for the TZ mid-bar (iTZh bars ago)
				}
				else
				{
					//Update TZADL for the current bar with the sub-data-series total
					if (CurrentBar == 0) TZAD.Set(dSubTotal);
					else TZAD.Set(TZAD[1] + dSubTotal);
					dSubTotal = 0;
				}
			}
			else if (!bSameAsChart)
			{
				//Sub-data series (bars used for calculation if not same as Main)
				if (CurrentBar < iTZLeft) return;
				else dSubTotal += iBarValue();  //Update TZADL sub-total
			}
				
        }

        #region Functions

		private int iBarValue()
		{
			//Return TZADL value for TZ bar being processed
			//Index [0] is current bar, [1] is previous bar, etc.

			//Calculate left and right highs and lows of potential transient zone (PTZ)
			double ldLeftLow = Low[iTZMidPrev];　
			double ldLeftHigh = High[iTZMidPrev];　
			for (int liX = iTZMidPrev + 1; liX <= iTZLeft; liX++)
			{
				if (Low[liX] < ldLeftLow) ldLeftLow = Low[liX];
				if (High[liX] > ldLeftHigh) ldLeftHigh = High[liX];
			}
			double ldRightLow = Low[iTZMidNext];　
			double ldRightHigh = High[iTZMidNext];　
			for (int liX = iTZMidNext - 1; liX >= iTZRight; liX--)
			{
				if (Low[liX] < ldRightLow) ldRightLow = Low[liX];
				if (High[liX] > ldRightHigh) ldRightHigh = High[liX];
			}

			//Calculate and return TZADL
			int liValue = 0;  //default (no TZ)

			//Check for Mid Down TZ (must be at least 2 pips high to contain a transient price)
			int liTZk = (int)Math.Round((ldRightHigh + TickSize - ldLeftLow) / TickSize);
			if (liTZk < 0)
			{
				//Mid Down TZ
				liValue = (bUseTZk ? liTZk : -1);
			}
			else
			{
				//Check for Mid Up TZ
				liTZk = (int)Math.Round((ldRightLow - TickSize - ldLeftHigh) / TickSize);
				if (liTZk > 0) liValue = (bUseTZk ? liTZk : 1);
			}

			return liValue;
				
		}

		private string sDateTime(DateTime atD)
		{
			//Return DateTime as a standard formatted string
			return atD.ToString("dd-MMM-yyyy HH:mm");
		}

		private void SetVars()
		{
			//First call (for either data series)
			//Accesses properties that are unavailable in Initialize()
			//Set vars
			if (iBarPeriod == 0 || (BarsPeriod.Id == PeriodType.Minute && BarsPeriod.Value == iBarPeriod)) bSameAsChart = true;
			else bSameAsChart = false;
			bFirst = false;
			dSubTotal = 0;
			//TZ bar indices (0 = current bar = rightmost bar of TZ)
			iTZMid = iTZh;
			iTZLeft = iTZMid + iTZh;
			iTZRight = 0;
			iTZMidPrev = iTZMid + 1;
			iTZMidNext = iTZMid - 1;
			
			//Write details to Output window
			ClearOutputWindow();
			Print(Instrument.FullName + ", Period = " + BarsPeriod.Id.ToString().Substring(0, 1) + BarsPeriod.Value + ", Pip Value = " + TickSize.ToString() +
				", Run time = " + sDateTime(DateTime.Now) + " (local)");
			Print("Bar calculation period = " + (bSameAsChart ? " same as chart bars" : "M" + iBarPeriod));
			Print("TZ width = " + iTZh + " bars either side of mid-bar");
			Print("Use TZ Height = " + bUseTZk);
		}

		#endregion
		
        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries TZAD
        {
            get { return Values[0]; }
        }

		//Parameters 
		[Description("Period of each bar (in minutes) on which the calculation is performed. 0 = same as chart bars.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("Bar calculation period")]
		public int BarPeriod
		{
			get { return iBarPeriod; }
			set { iBarPeriod = Math.Max(0, value); }
		}

		[Description("Width of transient zone (in bars) each side of the mid-bar. Values > 1 will measure stronger TZs, but will increase lag.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("TZ width")]
		public int TZWidth
		{
			get { return iTZh; }
			set { iTZh = Math.Max(1, value); }
		}
		
		[Description("True = add or subtract the TZ's height (in pips) to the running total. False = always add or subtract 1 for each TZ.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("Use TZ height")]
		public bool UseTZHeight
		{
			get { return bUseTZk; }
			set { bUseTZk = value; }
		}

		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TZADL[] cacheTZADL = null;

        private static TZADL checkTZADL = new TZADL();

        /// <summary>
        /// TZADL: an adaption of Chaikin's Accumulation/Distribution Line, which is based on mid-bar transient zones rather than volume.
        /// </summary>
        /// <returns></returns>
        public TZADL TZADL(int barPeriod, int tZWidth, bool useTZHeight)
        {
            return TZADL(Input, barPeriod, tZWidth, useTZHeight);
        }

        /// <summary>
        /// TZADL: an adaption of Chaikin's Accumulation/Distribution Line, which is based on mid-bar transient zones rather than volume.
        /// </summary>
        /// <returns></returns>
        public TZADL TZADL(Data.IDataSeries input, int barPeriod, int tZWidth, bool useTZHeight)
        {
            if (cacheTZADL != null)
                for (int idx = 0; idx < cacheTZADL.Length; idx++)
                    if (cacheTZADL[idx].BarPeriod == barPeriod && cacheTZADL[idx].TZWidth == tZWidth && cacheTZADL[idx].UseTZHeight == useTZHeight && cacheTZADL[idx].EqualsInput(input))
                        return cacheTZADL[idx];

            lock (checkTZADL)
            {
                checkTZADL.BarPeriod = barPeriod;
                barPeriod = checkTZADL.BarPeriod;
                checkTZADL.TZWidth = tZWidth;
                tZWidth = checkTZADL.TZWidth;
                checkTZADL.UseTZHeight = useTZHeight;
                useTZHeight = checkTZADL.UseTZHeight;

                if (cacheTZADL != null)
                    for (int idx = 0; idx < cacheTZADL.Length; idx++)
                        if (cacheTZADL[idx].BarPeriod == barPeriod && cacheTZADL[idx].TZWidth == tZWidth && cacheTZADL[idx].UseTZHeight == useTZHeight && cacheTZADL[idx].EqualsInput(input))
                            return cacheTZADL[idx];

                TZADL indicator = new TZADL();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.BarPeriod = barPeriod;
                indicator.TZWidth = tZWidth;
                indicator.UseTZHeight = useTZHeight;
                Indicators.Add(indicator);
                indicator.SetUp();

                TZADL[] tmp = new TZADL[cacheTZADL == null ? 1 : cacheTZADL.Length + 1];
                if (cacheTZADL != null)
                    cacheTZADL.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheTZADL = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// TZADL: an adaption of Chaikin's Accumulation/Distribution Line, which is based on mid-bar transient zones rather than volume.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TZADL TZADL(int barPeriod, int tZWidth, bool useTZHeight)
        {
            return _indicator.TZADL(Input, barPeriod, tZWidth, useTZHeight);
        }

        /// <summary>
        /// TZADL: an adaption of Chaikin's Accumulation/Distribution Line, which is based on mid-bar transient zones rather than volume.
        /// </summary>
        /// <returns></returns>
        public Indicator.TZADL TZADL(Data.IDataSeries input, int barPeriod, int tZWidth, bool useTZHeight)
        {
            return _indicator.TZADL(input, barPeriod, tZWidth, useTZHeight);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// TZADL: an adaption of Chaikin's Accumulation/Distribution Line, which is based on mid-bar transient zones rather than volume.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TZADL TZADL(int barPeriod, int tZWidth, bool useTZHeight)
        {
            return _indicator.TZADL(Input, barPeriod, tZWidth, useTZHeight);
        }

        /// <summary>
        /// TZADL: an adaption of Chaikin's Accumulation/Distribution Line, which is based on mid-bar transient zones rather than volume.
        /// </summary>
        /// <returns></returns>
        public Indicator.TZADL TZADL(Data.IDataSeries input, int barPeriod, int tZWidth, bool useTZHeight)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TZADL(input, barPeriod, tZWidth, useTZHeight);
        }
    }
}
#endregion
